SOURCE_DATA_COLUMN_INFO = "source_data_column_info"
FEATURE_COLUMN_INFO = "feature_column_info"
FEATURE_STORE = "feature_store"
INPUT_COLUMNS = "input_columns"
NAME = "name"
OUTPUT_NAME = "output_name"
SOURCE = "source"
TRAINING_DATA = "training_data"
INPUT_TABLES = "input_tables"
TABLE_NAME = "table_name"
TABLE_ID = "table_id"
SERIALIZATION_VERSION = "serialization_version"
