# Constants defining feature table property names.
PRIMARY_KEYS = "databricks.feature_store.primary_keys"
PARTITION_COLUMNS = "databricks.feature_store.partition_columns"
DESCRIPTION = "databricks.feature_store.description"
ONLINE_STORES = "databricks.feature_store.online_stores"

ALL_PROPERTIES = [PRIMARY_KEYS, PARTITION_COLUMNS, DESCRIPTION, ONLINE_STORES]
